
# ==============================================================================
#  Log-likelihood function for 2-level binary model (matrix notation)
# ==============================================================================
#
loglikBin<- function(par,be,rugen,y,X,R,C=diag(ncol(X)),N,logscala=1)
{
  require(magrittr)

  likest<- NULL
  
  for (j in 1:N)
  {
  	rugen(par) %>% condens(par,be,.,y,X,R,C,logscala) %>% c(likest) -> likest
  }
  
  likest %>% mean %>% log %>% return	
}
# ==============================================================================



# ==============================================================================
#  Hessian
# ==============================================================================

hessianBin<- function(par,be,rugen,y,X,R,C=diag(ncol(X)),N,dpar)
{
  f<- function(x) { loglikBin(x,be,rugen,y,X,R,C,N) }
  e<- function(i,n) { out<- rep(0,n); out[i]<- 1; out }
  k<- length(par)
  out<- matrix(0,k,k)
  f0<- f(par)
  
  for (i in 1:k)
  {
  	for (j in 1:i)
  	{
  	  {f(par+dpar*(e(i,k)+e(j,k)))-f(par+dpar*e(i,k))-f(par+dpar*e(j,k))+f0} %>%
  	    divide_by(sum(dpar*e(i,k))*sum(dpar*e(j,k))) -> out[i,j]
  	}
  }
  
  lower.tri(out) %>% {.*out+(1-.)*t(out)} %>% return
}
# ==============================================================================



# ==============================================================================
#  Conditioned log-density function for 2-level binary model (matrix notation)
# ==============================================================================

logcondens<- function(par,be,U,y,X,R,C)
{
  {-X%*%be(par)-rowSums(X*(R%*%(U%*%C)))} %>%	# compute the exponent
    exp %>% add(1) %>% {1/.} %>%					# compute the probabilities
    {.^y*(1-.)^(1-y)} %>%						# compute the marginal probabilities
    log %>% sum %>% return						# compute the density
}
# ==============================================================================


# ==============================================================================
#  Conditioned log-translated log-density function for 2-level binary model (matrix notation)
# ==============================================================================

condens<- function(par,be,U,y,X,R,C,logscala=1)
{
  {-X%*%be(par)-rowSums(X*(R%*%(U%*%C)))} %>%			# compute the exponent
    exp %>% add(1) %>% {1/.} %>%							# compute the probabilities
    {.^y*(1-.)^(1-y)} %>%								# compute the marginal probabilities
    log %>% sum %>% add(logscala) %>% exp %>% return		# compute the density
}
# ==============================================================================




# ==============================================================================
# Simulation
# ==============================================================================
#
# Input:
#
# X  - matrix (n x k1) of regressors
# par - vector of ...
# be - vector (k x 1) of coefficients
# Om - covariance matrix (G*k2 x G*k2) of 2nd-level random effects U (G x k2)
# R  - matrix (n x G) which describes the multilevel structure of units
# C  - matrix (k2 x k1) which assigns the random effects to the regressors
#
# Output:
# A vector with the simulated dependent variable

simBin<- function(par,be,rugen,X,R,C)
{
  require(magrittr)

  rugen(par) %>%									# extract U
    {-X%*%be(par)-rowSums(X*(R%*%(.%*%C)))} %>% # compute the exponent
    exp %>% add(1) %>% {1/.} %>%					# compute the probabilities
    rbinom(nrow(X),1,.) %>% return				# simulate "y"
}
# ==============================================================================


