
library(magrittr)
#library(Matrix)
#library(numDeriv)

rm(list=ls())

set.seed(1861)

source('loglikBin.R')
source('noisyCE.R')

# Piano di simulazione
linux_id<- '380171'
cartella<- 'superficie_N-rho/sup_050'
piano<- expand.grid('N'=rep(c(100,180,320,560),3)*rep(c(1,10,100),each=4),'rho'=(1:10)/100,'asmooth'=1,'bsmooth'=0.975,'qsmooth'=10,
  'pval'=0.1,'vbuff'=0.5,'pgeweke1'=0.3,'pgeweke2'=0.4,'buff'=20)
piano$nomef<- paste0(linux_id,'_',sprintf('%06d',1:nrow(piano)))
attr(piano,'out.attrs')<- NULL
save(piano,file=paste0(cartella,'/piano_',linux_id,'.RData'))

# Modello
n<- 250
G<- 3
k1<- 4
reor<- 1:k1 	# reor: random effects on regressors
k2<- length(reor)
theta<- c(1:k1,0.1)
names(theta)<- c(paste0('beta',0:(k1-1)),'sigma2')


# Generazione dati
# Data
dati<- data.frame(num=1:n,g=sample(1:G,n,TRUE))
# Matrix of regressors
{rnorm(n*(k1-1),,1)} %>% matrix(n,k1-1) %>% cbind(1,.) ->X
colnames(X)<- paste0('X',0:(ncol(X)-1))
dati %<>% cbind(X)
# Matrix "R"
dati %>% {table(.$n,.$g)} %>% unname ->R
# Matrix "C"
matrix(0,k2,k1) %>% {.[,reor]<- diag(1,k2); .} ->C
# Generator process of random effects
rugen<- function(x) { rnorm(G*k2,,x[k1+1]) %>% matrix(G,k2) %>% return }
# Function extracting beta coefficients from parameters vector
be<- function(par) { par[1:k1] }

# "Randomize"
set.seed(NULL)

# Draw...
seme<- .Random.seed
y<- simBin(theta,be,rugen,X,R,C)
S<- function(x,u) { logcondens(x,be,u,y,X,R,C) }


# Fit the models
for (j in 1:nrow(piano))
{
	cat('Grid point ',j,'...',sep='')
	system.time(
		out<- noisyCE(S,rugen,v0=c(rep(c(0,10),k1),-2.31,2.15),domain=c(rep(0,k1),1),N=piano$N[j],rho=piano$rho[j],
		asmooth=piano$asmooth[j],bsmooth=piano$bsmooth[j],qsmooth=piano$qsmooth[j],
		pval=piano$pval[j],buff=piano$buff[j],vbuff=piano$vbuff[j],pgeweke=c(piano$pgeweke1[j],piano$pgeweke2[j]))
	)[3]-> tempi
	out$tempo<- unname(tempi)
	save(out,y,seme,file=paste0(cartella,'/',piano$nomef[j],'.RData'))
	cat(' done!\n')
}


