
library(magrittr)
library(Matrix)
library(coda)
library(numDeriv)
library(lme4)



rm(list=ls())

source('loglikBin.R')
source('noisyCE.R')


derifun<- function(x0,Obj,i,N,logscala=852)
{
	filtro<- rep(0,length(x0))
	filtro[i]<- 1
	margi<- function(x) { loglikBin(x*filtro+x0*(1-filtro),be=be,rugen=rugen,y=y,X=X,R=R,C=C,N=1,logscala=logscala) }
	xH<- rnorm(N,xast[i],10*Obj$estimate[2*i])
	yH<- NULL
	for (j in xH) { yH %<>% c(margi(j)) }
	data.frame(x=xH-x0[i],y=yH) %>% {.[is.finite(.$y),]} %>% lm(y~poly(x,2),data=.) %>% coef %>% extract(3) %>% multiply_by(2) %>% return
}


fitted<- function(Obj) { Obj$estimate[2*(1:k1)-1] %>% matrix %>% {X%*%.} %>% return }
inv.logit<- function(x) { 1/(1+exp(-x)) }
logit<- function(x) { log(x/(1-x))}

# Risultati
risultati<- function(Obj,conf=0.95,arr=4)
{
	require(magrittr)
	df<- n-length(Obj$estimate)/2
	out<- data.frame(coef=Obj$estimate[2*(1:(length(Obj$estimate)/2))-1])
	rownames(out)<- names(Obj$estimate)[2*(1:(length(Obj$estimate)/2))-1]
	Obj %>% use_series(covout) %>% diag %>% sqrt -> out$se
	out[,paste0('low-',conf*100,'%')]<- out$coef+out$se*qt((1-conf)/2,df)
	out[,paste0('high-',conf*100,'%')]<- out$coef-out$se*qt((1-conf)/2,df)
	out[,'t-stat']<- abs(out$coef/out$se)
	out[,'p-value']<- 2*pt(out[,'t-stat'],df,lower.tail=FALSE)
	out[,7]<- ''
	out[which(out[,6]<0.1  ),7] <- '.  '
	out[which(out[,6]<0.05 ),7] <- '*  '
	out[which(out[,6]<0.01 ),7] <- '** '
	out[which(out[,6]<0.001),7] <- '***'
	colnames(out)[7]<- ''
	out<- list(coef=out,loglik=NA,Rsq=NA,df=df,n=n,npar=n-df)
	pout<- out$coef
	pstat<- matrix(c(out$npar,out$n,out$df,out$loglik,out$Rsq*100),dimnames=list(c('Number of parameters:','Sample size:','Degrees of freedom:','Log-lik:','R^2 (max: 100):'),''))
	if (!is.null(arr)) { pout[,1:6] %<>% round(arr) }
	(39+6*arr) %>% rep('-',.) %>% paste0(collapse='') %>% cat('\n',sep='')
	print(pout,quote=FALSE)
	print(round(pstat,2))
	(39+6*arr) %>% rep('-',.) %>% paste0(collapse='') %>% cat('\n',sep='')
	if (is.null(Obj$message)) { msg<- Obj$code } else { msg<- paste(Obj$code,'-',Obj$message) }
	cat('Algorithm convergence code:',msg,'\n')
	invisible(out)
}



load('Campione.RData')

# Modello
dati %>% extract(,1) ->y
dati %>% model.matrix(~avemp+degree+gdp,data=.) ->X
dati %>% nrow ->n
dati %>% use_series(idReg) %>% unique %>% length ->G
X %>% ncol -> k1
1:k1 ->reor 	# reor: random effects on regressors
reor %>% length ->k2
# Matrice R
matrix(0,n,G) ->R
R[cbind(1:n,as.numeric(dati$idReg))]<- 1
# Matrice C
matrix(0,k2,k1) %>% {.[,reor]<- diag(1,k2); .} ->C

# Generator process of random effects
rugen<- function(x) { rnorm(G*k2,0,1) %>% matrix(G,k2) %>% multiply_by_matrix(diag(x[-(1:k1)])) %>% return }
# Integrand of the objective function
S<- function(x,u) { logcondens(x,be,u,y,X,R,C) }
# Function extracting beta coefficients from parameters vector
be<- function(par) { par[1:k1] }



# Modello non multilevel
mod0<- glm(y~avemp+degree+gdp,family='binomial',data=dati)



# AGQ m=1
#glmer(cbind(y,1-y)~avemp+degree+gdp+(1+avemp+degree+gdp|idReg),data=dati,family='binomial',nAGQ=1) ->modAGQ1



# Entropia relativa (N=1000)
cat('  Cross-entropy Estimation (N=1000) ...     ')
tempi<- numeric(2)
system.time(
	ce1000<- noisyCE(S,rugen,v0=c(0,10,0,0.1,rep(c(0,10),k1-2),rep(c(-2.31,2.15),k2)),domain=c(rep(0,k1),rep(1,k2)),
		N=10000,rho=0.04,asmooth=1,bsmooth=0.975,qsmooth=10,pval=0.05,buff=20,vbuff=0.5,pgeweke=c(0.3,0.4))
)[3]-> tempi[1]
ce1000$tempo<- unname(tempi)
cat(' [  ok  ]\n')

cat('  Estimation of the Hessian ...             ')
covout<- NULL
xast<- ce1000$estimate[2*(1:(length(ce1000$estimate)/2))-1]
for (i in 1:length(xast)) { xast %>% derifun(ce1000,i,1000) %>% c(covout,.) ->covout }
covout %>% {-1/.} %>% diag ->ce1000$covout
#covout %>% {-1/.} %>% diag %>% nearPD %T>% {cat('Frobenius:',.$normF,'\n')} %>% use_series(mat) ->out$covout
names(ce1000$estimate)[2*(1:(length(ce1000$estimate)/2))-1]<- c(colnames(X),paste0('sigma_',colnames(X)[reor]))
cat(' [  ok  ]\n')



# Entropia relativa (N=10000)
cat('  Cross-entropy Estimation (N=10000) ...    ')
tempi<- numeric(2)
system.time(
	ce10000<- noisyCE(S,rugen,v0=c(0,10,0,0.1,rep(c(0,10),k1-2),rep(c(-2.31,2.15),k2)),domain=c(rep(0,k1),rep(1,k2)),
		N=10000,rho=0.04,asmooth=1,bsmooth=0.975,qsmooth=10,pval=0.05,buff=20,vbuff=0.5,pgeweke=c(0.3,0.4))
)[3]-> tempi[1]
ce10000$tempo<- unname(tempi)
cat(' [  ok  ]\n')

cat('  Estimation of the Hessian ...             ')
covout<- NULL
xast<- ce10000$estimate[2*(1:(length(ce10000$estimate)/2))-1]
for (i in 1:length(xast)) { xast %>% derifun(ce10000,i,1000) %>% c(covout,.) ->covout }
covout %>% {-1/.} %>% diag ->ce10000$covout
#covout %>% {-1/.} %>% diag %>% nearPD %T>% {cat('Frobenius:',.$normF,'\n')} %>% use_series(mat) ->out$covout
names(ce10000$estimate)[2*(1:(length(ce10000$estimate)/2))-1]<- c(colnames(X),paste0('sigma_',colnames(X)[reor]))
cat(' [  ok  ]\n')



cat('\n\n')
cat('Logit regression:\n')
70 %>% rep('-',.) %>% paste0(collapse='') %>% cat('\n',sep='')
mod0 %>% summary %>% print
#cat('\nMultilevel logit regression (AGQ, m=1):\n')
#70 %>% rep('-',.) %>% paste0(collapse='') %>% cat('\n',sep='')
#modAGQ1 %>% summary %>% print
cat('\nMultilevel logit regression (CE, N=1000):\n')
ce1000 %>% risultati(arr=5)
cat('\nMultilevel logit regression (CE, N=10000):\n')
ce10000 %>% risultati(arr=5)




